import {Plugin} from 'ckeditor5/src/core';
import SpanCommand from "./spancommand";
import SpanRemoveCommand from "./spanremovecommand";
import {  getAttrsData } from "./utils";



const SPAN = 'span';

export default class SpanEditing extends Plugin {
  init() {
    this._defineCommands();
    this._defineSchema();
    this._defineConverters();
  }

  _defineCommands() {
    this.editor.commands.add(
      'addSpan', new SpanCommand(this.editor, SPAN)
    );
    this.editor.commands.add(
      'removeSpan', new SpanRemoveCommand(this.editor, SPAN)
    );
  }

  _defineSchema() {
    const schema = this.editor.model.schema;
    const model = this.editor.model;
    //https://ckeditor.com/docs/ckeditor5/latest/api/module_engine_model_schema-SchemaItemDefinition.html#member-inheritAllFrom
    schema.register(SPAN, {
      allowIn: [ '$root', '$container', '$block', '$text' ],
      allowChildren: ['$text',  '$inlineObject' ],
      isBlock: true,
      isInline: true,
      isContent: true,
      allowAttributes: [SPAN, 'class', 'title', 'id', 'lang', 'style'],
    });
    model.schema.extend( '$text', { allowIn: 'span', allowAttributes: ['class', 'title', 'id', 'lang', 'style'], } );
  }

  _defineConverters() {
    const editor = this.editor;
    //https://ckeditor.com/docs/ckeditor5/latest/api/module_engine_conversion_conversion-Conversion.html#function-elementToElement
    //https://ckeditor.com/docs/ckeditor5/latest/api/module_engine_conversion_conversion-Conversion.html#function-attributeToElement
    /*
        editor.conversion.elementToElement(
          { model: SPAN, view: 'span', title: 'Span', class: 'ck-span', id:'', lang:"", style:"" }      
        );
    */


    editor.conversion.for("downcast").elementToElement({
      model: SPAN,
      view: (modelElement, { writer }) => {
        const attr = getAttrsData(modelElement);
        
        const container = writer.createContainerElement("span", attr);
        return container;
      },
    });

    editor.conversion.for( 'upcast' ).elementToElement( {
      //model: SPAN,
      model: ( viewElement, { writer } ) => {
				// Do not auto-paragraph empty elements.
				if ( viewElement.isEmpty ) {
					//return null;
				}
        
				return writer.createElement( SPAN );
			},
      view: {
        name: 'span',
        title: 'Span', class: 'ck-span', id:'', lang:"", style:""
      },
    });


    /*
    editor.conversion.elementToElement( {
      model: SPAN,
      view: {
        name: 'span',
        title: 'Span', class: 'ck-span', id:'', lang:"", style:""
      },
      converterPriority: 'high'
    } );
     */

		const blockQuoteCommand = this.editor.commands.get( 'addSpan' );
		const selection = this.editor.model.document.selection;
    const viewDocument = this.editor.editing.view.document;
		const enterCommand = editor.commands.get( 'enter' );

    		// Overwrite default Enter key behavior.
		// If Enter key is pressed with selection collapsed in empty block inside a quote, break the quote.
		this.listenTo( enterCommand, 'afterExecute', ( evt, data ) => { 

      const positionParent = selection.getFirstPosition().parent;
      const isSpan = positionParent.is( 'element', SPAN );
      //console.log('removeSpan isSpan=', isSpan, 'parent=', positionParent.is( 'element', 'paragraph' ), 'childCount', positionParent.childCount)
      if ( isSpan && !positionParent.is( 'element', 'paragraph' ) /*&& positionParent.childCount === 0*/ ) {
        //data.writer.rename( positionParent, 'paragraph' );
        //data.writer.remove( positionParent );
        data.writer.unwrap( positionParent  );
      }
		}, { context: 'span' } );


  }
}